import _ from 'lodash'

import { TOCProvider, Component } from 'substance'

class NotesProvider extends TOCProvider {
  computeEntries () {
    const docMain = this.getDocument()
    const nodesMain = docMain.getNodes()

    const docMini = this.config.miniEditorSession.document
    let nodesMini = ''
    if (docMini) nodesMini = docMini.getNodes()

    if (docMini)console.log('paragraphs', nodesMini)

    // get all notes from the document
    const notesMain = _.pickBy(nodesMain, function (value, key) {
      return value.type === 'note'
    })
    const notesMini = _.pickBy(nodesMini, function (value, key) {
      return value.type === 'isolated-note'
    })

    const entriesMain = this.sortNodes(notesMain)
    const entriesMini = this.sortNodesMini(notesMini)

    // console.log(entriesMain, entriesMini)
    const entries = {
      main: entriesMain,
      mini: entriesMini
    }

    if (entriesMini && entriesMain.length < entriesMini.length) {
      this.findDeletedNote(entries)
    }
    return entries
  }

  sortNodes (nodes) {
    let notes = _.clone(nodes)
    const doc = this.getDocument()
    const container = doc.get('body')

    // sort notes by
    //   the index of the containing block
    //   their position within that block

    notes = _.map(notes, function (note) {
      const blockId = note.path[0]
      const blockPosition = container.getPosition(blockId)
      const nodePosition = note.start.offset

      return {
        id: note.id,
        'note-content': note['note-content'],
        blockPosition: blockPosition,
        nodePosition: nodePosition
      }
    })

    return _.sortBy(notes, ['blockPosition', 'nodePosition'])
  }

  sortNodesMini (nodes) {
    if (this.config.miniEditorSession === '') return

    let notes = _.clone(nodes)
    let rootEl = this.config.miniEditorContext.editor.el

    notes = _.map(notes, function (note) {
      const element = Component.unwrap(rootEl.find('p[data-path="' + note.id + '.content"]'))
      // removes a <br> insterted at the end by inline node
      const text = element.innerHTML.slice(0, -4)
      let noteContent = '<isolated-note data-id="' + note.id + '" parent-id="' + note.parentNoteId + '" data-type="isolated-note">' + text + '</isolated-note>'

      return {
        id: note.id,
        content: noteContent,
        parentNoteId: note.parentNoteId
      }
    })

    return notes
  }

  showNote (note) {
    const notes = this.computeEntries()
    const miniNote = _.find(notes.mini, ['parentNoteId', note.id])
    if (miniNote) this.config.miniEditorContext.editor.emit('noteSelected', miniNote.id)
  }

  createNodeData (note) {
    return {
      'type': 'isolated-note',
      'content': note['note-content'],
      'parentNoteId': note.id
    }
  }

  createIsolatedNote (note) {
    const notes = this.computeEntries()

    const containerId = this.getMiniContainerId()
    let surface = this.config.miniEditorContext.surfaceManager.getSurface(containerId)
    if (notes.mini && notes.mini.length > 0) {
      let container = surface.getContainer()

      let findIndex = _.findIndex(notes.main, ['id', note.id])

      let nodeData = this.createNodeData(note)
      this.config.miniEditorSession.transaction(function (tx) {
        if (!tx.get(nodeData.id)) {
          let blockNode = tx.create(nodeData)
          tx.update(container.getContentPath(), { type: 'insert', pos: findIndex, value: blockNode.id })
        }
      })
    } else {
      surface.createText()
      this.config.miniEditorSession.transaction(function (tx) {
        let nodeData = this.createNodeData(note)
        tx.insertBlockNode(nodeData)
      }.bind(this))
    }
  }

  findDeletedNote (entries) {
    let position
    let note
    if (entries.main.length === 0) {
      position = 0
      note = entries.mini[0]
    } else {
      _.forEach(entries.main, function (mainNote) {
        _.forEach(entries.mini, function (miniNote) {
          if (miniNote.parentNoteId !== mainNote.id) {
            position = _.findIndex(entries.mini, ['id', miniNote.id])
            note = miniNote
          }
        })
      })
    }

    this.removeIsolatedNote(position, note)
  }

  removeIsolatedNote (position, note) {
    const containerId = this.getMiniContainerId()
    let surface = this.config.miniEditorContext.surfaceManager.getSurface(containerId)
    let container = surface.getContainer()
    this.config.miniEditorSession.transaction(function (tx) {
      tx.update(container.getContentPath(), { type: 'delete', pos: position })
      tx.delete(note.id)
    })
  }

  getMiniContainerId () {
    return this.config.miniEditorContext.editor.props.containerId
  }
}

NotesProvider.tocTypes = ['note']

export default NotesProvider
